////////////////////////////////////////////////////////////////////////////////
//                                  Include                                   //
////////////////////////////////////////////////////////////////////////////////


#include "Oracle.h"


////////////////////////////////////////////////////////////////////////////////
//                              Public: Oracle                                //
////////////////////////////////////////////////////////////////////////////////


Oracle::Oracle(char* filename) : x(env), u(env), a(env), r(env), cx(env), 
				 obj(env), model(env), cplex(env)
{
  try {
    //----- User: Create data model -------------------------------------------1
    
    // Initiate variables
    IloInt i, j;

    // Data file
    ifstream file(filename);

    // m (rows), n (cols)
    file >> m >> n;

    // x (primal variables)
    x.add(IntVarMatrix(env, m));
    for (i = 0; i < m; i++)
      x[i] = IloIntVarArray(env, n, 0, 10);

    // u (dual multipliers)
    u.add(IloNumArray(env, m));

    // c (cost of allocating job j to agent i)
    IntMatrix c(env, m);
    for (i = 0; i < m; i++) {
      c[i] = IloIntArray(env, n);
      for (j = 0; j < n; j++)
        file >> c[i][j];
    }

    // a (resource consumed in allocating job j to agent i)
    a.add(IntMatrix(env, m));
    for (i = 0; i < m; i++) {
      a[i] = IloIntArray(env, n);
      for (j = 0; j < n; j++)
        file >> a[i][j];
    }

    // r (resource capacity of agent i)
    r.add(IloIntArray(env, m));
    for (i = 0; i < m; i++)
      file >> r[i];

    // model: object
    for (i = 0; i < m; i++)
      for (j = 0; j < n; j++)
        cx += c[i][j] * x[i][j];

    // model: lagrange relaxed cover constraint
    IloExpr lag(env);

    // model: maximize
    obj = IloMaximize(env, cx + lag);
    model.add(obj);    

    // model: cover constraint
    for (j = 0; j < n; j++) {
      IloExpr temp(env);
      for (i = 0; i < m; i++)
        temp += x[i][j];
      model.add(temp == 1);
      temp.end();
    }

    // model: x constraints
    for (i = 0; i < m; i++)
      for (j = 0; j < n; j++) {
        model.add(x[i][j] >= 0);
        model.add(x[i][j] <= 1);
      }

    // cplex: extract model
    cplex.extract(model);

    //-------------------------------------------------------------------------1

    //----- User: Variables that must be set ----------------------------------2

    // u size
    uSize = m;

    // x dimensions and size
    xDims = 2;
    xDim1Size = m;
    xDim2Size = n;

    // Add all unconstrained multipliers u, end with an InINF
    ucVector = new unsigned int[1];
    ucVector[0] = InINF;

    // Is the primal problem of min type
    isMinProblem = false;

    //-------------------------------------------------------------------------2

    iteration = 0;
   
  } catch(IloException& e) {
    cerr << "Error: " << e.getMessage() << endl;
  } catch(...) {
    cout << "Error" << endl;
  }
}


////////////////////////////////////////////////////////////////////////////////
//                              Public: ~Oracle                               //
////////////////////////////////////////////////////////////////////////////////


Oracle::~Oracle()
{
  env.end();
}


////////////////////////////////////////////////////////////////////////////////
//                               Public: Solve                                //
////////////////////////////////////////////////////////////////////////////////


void Oracle::Solve(const double uVector[], double &fValue, double sgVector[], 
		   double xVector[])
{
  //----- User: Solve model ---------------------------------------------------1
  
  // Inititate variables
  IloInt i, j;
  IloNum sum;

  // Update u
  u.clear();
  for (i = 0; i < m; i++)
    u.add(uVector[i]);

  // Update model
  model.remove(obj);
  
  IloExpr lag(env);
  for (i = 0; i < m; i++) {
    IloExpr temp(env);
    for (j = 0; j < n; j++)
      temp += a[i][j] * x[i][j];
    lag += u[i] * (r[i] - temp);
    temp.end();
  }

  obj = IloMaximize(env, cx + lag);
  lag.end();

  model.add(obj);

  // Solve model
  cplex.solve();

  // fValue (with "-" when min problem)
  fValue = cplex.getValue(obj);

  // sgVector (with "-" when min problem)
  for (i = 0; i < m; i++) {
    sum = 0;
    for (j = 0; j < n; j++)
      sum += a[i][j] * cplex.getValue(x[i][j]);
    sgVector[i] = (r[i] - sum);
  }

  // xVector
  for (i = 0; i < m; i++)
    for (j = 0; j < n; j++)
      xVector[i * j + j] = cplex.getValue(x[i][j]);

  // Print iteration
  iteration += 1;
  cout << endl;
  cout << "Iteration " << iteration << endl;
  /*
  cout << "------------------------------" << endl;
  cout << "   f = " << fValue << endl;
  cout << "  cx = " << cplex.getValue(cx) << endl;
  for (i = 0; i < m; i++)
    cout << "  u" << i + 1 << " = " << u[i] << endl;
  for (j = 0; j < n; j++)
    cout << "  x" << j + 1 << " = " << cplex.getValue(x[0][j]) << endl;
  for (i = 0; i < m; i++)
    cout << "  g" << i + 1 << " = " << sgVector[i] << endl;
  cout << "------------------------------" << endl;
  cout << endl;
  */
  //---------------------------------------------------------------------------1
}


////////////////////////////////////////////////////////////////////////////////
//                                Public: getX                                //
////////////////////////////////////////////////////////////////////////////////


long Oracle::getUSize()
{
  return uSize;
}

long Oracle::getXDims()
{
  return xDims;
}

long Oracle::getXDim1Size()
{
  return xDim1Size;
}

long Oracle::getXDim2Size()
{
  return xDim2Size;
}

bool Oracle::getIsMinProblem()
{
  return isMinProblem;
}

unsigned int* Oracle::getUCVector()
{
  return ucVector;
}


////////////////////////////////////////////////////////////////////////////////
